# Generated by Django 4.0.5 on 2022-09-18 04:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0005_tag_tagtaskmapping'),
    ]

    operations = [
        migrations.AddField(
            model_name='tag',
            name='status',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



from django.contrib import admin

# Register your models here.
from .models import Appuser, Task


@admin.register(Task)
class TaskAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'description', 'priority', 'create_date',
                    'due_date', 'modified_date', 'status', 'created_by')
    list_filter = ('priority', 'create_date', 'status', 'created_by')
    search_fields = ('name', 'description')


@admin.register(Appuser)
class AppuserAdmin(admin.ModelAdmin):
    list_filter = ('username', 'first_name')



from django.contrib import admin
from django.urls import path, include
from .views import *

from rest_framework.routers import DefaultRouter
from rest_framework.authtoken import views


router = DefaultRouter()
router.register(r'tasks', TaskList, basename='tasks')
router.register(r'tags', TagList, basename='tags')
router.register('auth', UserViewSet, basename='auth')
urlpatterns = [
    path('login', views.obtain_auth_token),
    path('', home, name='home'),
    path('', include(router.urls)),
    path('tasks_statistics/', tasks_statistics, name='tasks_statistics'),
    path('check_due_task/', check_due_task, name='check_due_task'),
]



# Generated by Django 4.0.5 on 2022-07-17 02:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='created_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from rest_framework import serializers
from .models import *


class TagtaskSerializer(serializers.ModelSerializer):
    class Meta:
        model = TagTaskMapping
        fields = '__all__'


class TaskSerializer(serializers.ModelSerializer):
    class Meta:
        model = Task
        fields = '__all__'

    def to_representation(self, instance):
        data = super().to_representation(instance)
        data['tags'] = TagtaskSerializer(
            TagTaskMapping.objects.filter(task_id=data['id']), many=True).data
        return data


class TagSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tag
        fields = '__all__'


class ApppuserSerializer(serializers.ModelSerializer):
    class Meta:
        model = Appuser
        fields = '__all__'
        extra_kwargs = {
            'password': {'write_only': True}
        }

    def create(self, validated_data):
        user = Appuser.objects.create(**validated_data)
        user.set_password(validated_data.get('password'))
        user.save()
        return user



# Generated by Django 4.0.5 on 2022-07-17 03:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0003_task_status'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='status',
            field=models.CharField(blank=True, default='New', max_length=100, null=True),
        ),
    ]



from functools import partial
from django.http import HttpResponse
from django.shortcuts import render
from rest_framework import status
from rest_framework.response import Response
from .serializers import *
from .models import *
from rest_framework.decorators import api_view, APIView
from rest_framework import generics, mixins, viewsets
from django.shortcuts import get_object_or_404
from rest_framework.permissions import IsAuthenticated, AllowAny
import datetime
from django.db.models import Q


# Create your views here.


def home(request):
    return HttpResponse('Welcome to my Task App')


@api_view(['GET'])
def tasks_statistics(request):
    try:
        queryset = Task.objects.filter(
            created_by=request.user)
        data = {
            "due": queryset.filter(status="Due").count(),
            "todo": queryset.filter(status="New").count(),
            "done": queryset.filter(status="Success").count()
        }
        return Response(data, status=status.HTTP_200_OK)
    except Exception as e:
        print(e)
        return Response({"message": "Error in fetching counts", "detail": str(e)}, status=status.HTTP_400_BAD_REQUEST)


@api_view(['GET'])
def check_due_task(request):
    try:
        print(datetime.datetime.now())
        queryset = Task.objects.filter(
            due_date__lt=datetime.datetime.now(), status="New")
        due_tasks = queryset.count()
        queryset.update(status="Due")
        queryset = Task.objects.filter(
            due_date__gt=datetime.datetime.now(), status="Due")
        due_tasks = queryset.count()
        queryset.update(status="New")
        data = {
            "due_tasks": due_tasks,
        }
        return Response(data, status=status.HTTP_200_OK)
    except Exception as e:
        print(e)
        return Response({"message": "Error in fetching counts", "detail": str(e)}, status=status.HTTP_400_BAD_REQUEST)


class TaskList(viewsets.ModelViewSet):
    queryset = Task.objects.all()
    serializer_class = TaskSerializer
    permission_classes = [IsAuthenticated]

    def list(self, request):
        query = request.GET.get('query')
        if query:
            queryset = Task.objects.filter(
                Q(name__icontains=query) | Q(description__icontains=query), created_by=request.user).order_by('status')
        else:
            queryset = Task.objects.filter(
                created_by=request.user).order_by('status')
        serialized = TaskSerializer(queryset, many=True)
        return Response(serialized.data, status=status.HTTP_200_OK)

    def create(self, request):
        serialized = TaskSerializer(data=request.data)
        if serialized.is_valid():
            serialized.save(created_by=request.user)
            tags = request.data.get("tag_ids", None)
            if tags:
                tag_objs = []
                for tag in tags:
                    tag_obj = TagTaskMapping(
                        tag_id=tag, task_id=serialized.data.get('id'))
                    tag_objs.append(tag_obj)
                TagTaskMapping.objects.bulk_create(tag_objs)
            return Response(serialized.data, status=status.HTTP_201_CREATED)
        else:
            return Response({"message": serialized.errors}, status=status.HTTP_400_BAD_REQUEST)

    def update(self, request, pk=None):
        task_obj = get_object_or_404(Task, pk=pk)
        serialized = TaskSerializer(task_obj, data=request.data, partial=True)
        if serialized.is_valid():
            serialized.save(created_by=request.user)
            tags = request.data.get("tag_ids", None)
            existing_tags = list(TagTaskMapping.objects.filter(
                task_id=serialized.data.get('id')).values_list("tag_id", flat=True))
            print('Incoming tags', tags)
            print('Existing tags', existing_tags)
            adding_tags = list(set(tags)-set(existing_tags))
            removing_tags = list(set(existing_tags)-set(tags))
            print("adding_tags", adding_tags, "removing_tags", removing_tags)
            if tags:
                tag_objs = []
                for tag in adding_tags:
                    tag_obj = TagTaskMapping(
                        tag_id=tag, task_id=serialized.data.get('id'))
                    tag_objs.append(tag_obj)
                TagTaskMapping.objects.bulk_create(tag_objs)
                TagTaskMapping.objects.filter(
                    tag_id__in=removing_tags).delete()
            return Response(serialized.data, status=status.HTTP_201_CREATED)
        else:
            return Response({"message": serialized.errors}, status=status.HTTP_400_BAD_REQUEST)


class TagList(viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer
    permission_classes = [IsAuthenticated]

    def list(self, request):
        query = request.GET.get('query')
        if query:
            queryset = Tag.objects.filter(
                Q(name__icontains=query) | Q(description__icontains=query), created_by=request.user).order_by('status')
        else:
            queryset = Tag.objects.filter(
                created_by=request.user).order_by('status')
        serialized = TagSerializer(queryset, many=True)
        return Response(serialized.data, status=status.HTTP_200_OK)

    def create(self, request):
        serialized = TagSerializer(data=request.data)
        if serialized.is_valid():
            serialized.save(created_by=request.user)
            return Response(serialized.data, status=status.HTTP_201_CREATED)
        else:
            return Response({"message": serialized.errors}, status=status.HTTP_400_BAD_REQUEST)


class UserViewSet(viewsets.ModelViewSet):
    queryset = Appuser.objects.all()
    serializer_class = ApppuserSerializer
    permission_classes = [AllowAny]



from email.policy import default
from unicodedata import name
from django.db import models
from django.contrib.auth.models import AbstractUser

# Create your models here.


class Task(models.Model):
    name = models.CharField(max_length=250)
    description = models.CharField(max_length=1000, null=True, blank=True)
    priority = models.IntegerField(default=5, null=True, blank=True)
    status = models.CharField(
        max_length=100, default="New", null=True, blank=True)
    due_date = models.DateTimeField(blank=True, null=True)
    created_by = models.ForeignKey(
        "Appuser", on_delete=models.CASCADE, null=True, blank=True)
    create_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)


"""
This file contains the Data Models for Appuser Tables
"""


user_status_choices = [('Active', 'Active'), ('Inactive', 'Inactive')]


class Appuser(AbstractUser):
    user_id = models.AutoField(primary_key=True)
    # org_id = models.ForeignKey(Organization, on_delete=models.CASCADE)
    # role_id = models.ForeignKey(Role, on_delete=models.CASCADE)
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)
    first_login = models.BooleanField(default=True)
    email = models.EmailField(unique=True)
    recovery_email_address = models.CharField(max_length=100, blank=True)
    status = models.CharField(
        max_length=255, choices=user_status_choices, default='Active')
    phone = models.CharField(blank=True, null=True, max_length=255)
    created_date = models.DateTimeField(auto_now_add=True)
    created_by = models.CharField(max_length=1255, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)
    modified_by = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return str(self.username)


class Tag(models.Model):
    name = models.CharField(max_length=255, blank=True)
    status = models.CharField(max_length=255, blank=True)
    used_count = models.IntegerField(null=True, blank=True)
    created_date = models.DateTimeField(
        auto_now_add=True, null=True, blank=True)
    modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)
    created_by = models.CharField(max_length=1255, blank=True)
    modified_by = models.CharField(max_length=255, blank=True)


class TagTaskMapping(models.Model):
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE)
    task = models.ForeignKey(Task, on_delete=models.CASCADE)
    created_date = models.DateTimeField(
        auto_now_add=True, null=True, blank=True)
    modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)



# Generated by Django 4.0.5 on 2022-09-18 04:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0004_alter_task_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255)),
                ('used_count', models.IntegerField(blank=True, null=True)),
                ('created_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(auto_now=True, null=True)),
                ('created_by', models.CharField(blank=True, max_length=1255)),
                ('modified_by', models.CharField(blank=True, max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='TagTaskMapping',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(auto_now=True, null=True)),
                ('tag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='api.tag')),
                ('task', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='api.task')),
            ],
        ),
    ]
